/*
 *	btSerial.c
 *
 *	G2 Capstone
 *	Created by: Nigel Himmelreich
 *	ECE 492 - Winter 2015 - University of Alberta
 *
 * btSerial.h
 * Bluetooth Serial Connection for DE0 Nano to MOD-BT
 *
 * See appnote for references
 *
 */

#include "btSerial.h"

void singleWrite(alt_u8 c, alt_up_rs232_dev * serial_bt) {
	//Declare variables
	alt_u32 write_FIFO;
	int result = 0;

	//Disable reading
	alt_up_rs232_disable_read_interrupt(serial_bt);

	//Check for space in the write FIFO before placing character
	write_FIFO = alt_up_rs232_get_available_space_in_write_FIFO(serial_bt);
	if (write_FIFO >= FIFO_SIZE_REQUIRED) {
		//Place character into FIFO for Bluetooth module
		result = alt_up_rs232_write_data(serial_bt, c);
		if (result != 0) {
			printf("Error: Character not placed in FIFO\n");
		}
	} else {
		printf("Error: FIFO full\n");
	}
	OSTimeDlyHMSM(0, 0, 0, 10);
}

void multiWrite(char data[], alt_up_rs232_dev * serial_bt) {
	//Declare variables
	int i = 0;
	int len = strlen(data);

	//For each item in the string (array of characters), send each character to be written
	while (i < len) {
		singleWrite(data[i], serial_bt);
		i++;
	}

	printf("Wrote: %s : to BT unit\n", data);
}

alt_up_rs232_dev * initDevice (char device[]) {
	alt_up_rs232_dev * serial_bt;
	serial_bt = alt_up_rs232_open_dev("/dev/uart_0");

	if (serial_bt == NULL){
		printf("Error: Couldn't open connection to the Bluetooth module\n");
		return 0;
	} else {
		printf("Success: Opened connection to the Bluetooth module\n");
		alt_up_rs232_enable_read_interrupt(serial_bt);
		return serial_bt;
	}
}

void setupModule(alt_up_rs232_dev * serial_bt, char name[], char pin[]) {
	//Set command mode (3 times)
	multiWrite("+++\r", serial_bt);
	multiWrite("+++\r", serial_bt);
	multiWrite("+++\r", serial_bt);

	//Set the name
	multiWrite(name, serial_bt);

	//Disable echo
	multiWrite("ATE0\r", serial_bt);

	//Allow auto connections
	multiWrite("AT+BTAUT=1,0\r", serial_bt);

	//Set Bluetooth pin
	multiWrite(pin, serial_bt);

	//Start BT server
	multiWrite("AT+BTSRV=1\r", serial_bt);
}
