/*
 *	btSerial.h
 *
 *	G2 Capstone
 *	Created by: Nigel Himmelreich
 *	ECE 492 - Winter 2015 - University of Alberta
 *
 * btSerial.h
 * Bluetooth Serial Connection for DE0 Nano to MOD-BT
 *
 * See appnote for references
 *
 */

#include <stdio.h>
#include <string.h>
#include "includes.h"
#include "altera_up_avalon_rs232.h"
#include "altera_up_avalon_rs232_regs.h"

/* Constants*/
#define FIFO_SIZE_REQUIRED	0x80
#define	MAX_VALUE_SIZE		15

/* Functions*/

/**
 * @brief Writes a single character to the serial Bluetooth device
 * 
 * @param c				Single character to be written
 * @param serial_bt		The serial bluetooth device connection structure
 **/
void singleWrite(alt_u8 c, alt_up_rs232_dev * serial_bt);


/**
 * @brief Writes a string of character to the serial Bluetooth device
 *
 * @param data[]		String to be written
 * @param serial_bt		The serial bluetooth device connection structure
 **/
void multiWrite(char data[], alt_up_rs232_dev * serial_bt);


/**
 * @brief Initializes the device
 *
 * @param device		Device name "/dev/xxxx"
 *
 * @return	alt_up_rs232_dev	Device connection variable
 */
alt_up_rs232_dev * initDevice (char device[]);


/**
 * @brief Setups the module using the commands necessary and sets name and pin of Bluetooth connections
 *
 * @param serial_bt		The serial Bluetooth device connection structure
 * @param name			The string of the name for the connection
 * @param pin			The string of the pin for the connection
 */
void setupModule(alt_up_rs232_dev * serial_bt, char name[], char pin[]);
