/*
 * pn532.h
 *
 *  Created on: 2014-02-23
 *      Author: Daniel
 */

/*
 * Modified by: Nigel Himmelreich for ECE 492 - Winter 2015 - Group 2
 *
 * Added additional methods and fixed for use with DE0 Nano and Adafruit Shield
 */

#ifndef PN532_H_
#define PN532_H_

#include <stdbool.h>
#include "I2C.h"
#include "alt_types.h"

#define COMM_DELAY							1000
#define RESET_DELAY							400
#define MAX_UID_LENGTH						10
#define PN532_RESET_DELAY					2

#define PN532_I2C_ADDR						0x48
#define PN532_SCL_BASE						I2C_SCL_BASE
#define PN532_SDA_BASE						I2C_SDA_BASE
#define PN532_RESET_BASE					NFC_RESET_BASE

#define PN532_MAX_DATA_SIZE					255
#define PN532_FRAME_OVERHEAD				8 // including frame identifier byte
#define PN532_MAX_FRAME_SIZE				262
#define PN532_ACK_SIZE						6
#define PN532_FRAME_HEADER_SIZE				5
#define PN532_FRAME_FOOTER_SIZE				2
#define PN532_ID_OFFSET						4
#define PN532_DATA_OFFSET					6
#define PN532_MAX_TARGETS					1
#define PN532_AUTH_RESPONSE_SIZE			12
#define PN532_BLOCK_RESPONSE_SIZE			26

#define PN532_PREAMBLE                      0x00
#define PN532_STARTCODE1                    0x00
#define PN532_STARTCODE2                    0xFF
#define PN532_ACKCODE1						0x00
#define PN532_ACKCODE2						0xFF
#define PN532_POSTAMBLE                     0x00
#define PN532_STATUS_OK						0x00

#define PN532_WAKEUP			 			0x55 //dummy command

#define PN532_HOSTTOPN532                   0xD4
#define PN532_PN532TOHOST                   0xD5
#define PN532_MIFARE_ISO14443A				0x00
#define PN532_ISO_B							0x03

#define PN532_NORMAL_MODE					0x01
#define PN532_VCARD_TIMEOUT					0x00
#define PN532_VCARD_IRQ_ON					0x01
#define PN532_VCARD_IRQ_OFF					0x00

#define PN532_WAKEUP_ENABLE					0x80
#define PN532_STATUS_READY					0x01

// PN532 Commands
#define PN532_COMMAND_DIAGNOSE              0x00
#define PN532_COMMAND_GETFIRMWAREVERSION    0x02
#define PN532_COMMAND_GETGENERALSTATUS      0x04
#define PN532_COMMAND_READREGISTER          0x06
#define PN532_COMMAND_WRITEREGISTER         0x08
#define PN532_COMMAND_READGPIO              0x0C
#define PN532_COMMAND_WRITEGPIO             0x0E
#define PN532_COMMAND_SETSERIALBAUDRATE     0x10
#define PN532_COMMAND_SETPARAMETERS         0x12
#define PN532_COMMAND_SAMCONFIGURATION      0x14
#define PN532_COMMAND_POWERDOWN             0x16
#define PN532_COMMAND_RFCONFIGURATION       0x32
#define PN532_COMMAND_RFREGULATIONTEST      0x58
#define PN532_COMMAND_INJUMPFORDEP          0x56
#define PN532_COMMAND_INJUMPFORPSL          0x46
#define PN532_COMMAND_INLISTPASSIVETARGET   0x4A
#define PN532_COMMAND_INATR                 0x50
#define PN532_COMMAND_INPSL                 0x4E
#define PN532_COMMAND_INDATAEXCHANGE        0x40
#define PN532_COMMAND_INCOMMUNICATETHRU     0x42
#define PN532_COMMAND_INDESELECT            0x44
#define PN532_COMMAND_INRELEASE             0x52
#define PN532_COMMAND_INSELECT              0x54
#define PN532_COMMAND_INAUTOPOLL            0x60
#define PN532_COMMAND_TGINITASTARGET        0x8C
#define PN532_COMMAND_TGSETGENERALBYTES     0x92
#define PN532_COMMAND_TGGETDATA             0x86
#define PN532_COMMAND_TGSETDATA             0x8E
#define PN532_COMMAND_TGSETMETADATA         0x94
#define PN532_COMMAND_TGGETINITIATORCOMMAND 0x88
#define PN532_COMMAND_TGRESPONSETOINITIATOR 0x90
#define PN532_COMMAND_TGGETTARGETSTATUS     0x8A

//ISO Commands
#define ISO_COMMAND_LENGTH					7

// diagnose tests
#define PN532_DIAGNOSE_COMMUNICATION		0x00

typedef struct
{
	alt_u8 ic;
	alt_u8 ver;
	alt_u8 rev;
	alt_u8 support;
} FirmwareVersion;

typedef struct
{
	alt_u8 targetNumber;
	alt_u8 idLen;
	alt_u8 id[12];
} TargetData;

void nfcBegin();
void commandReadDataBlock(alt_u8 targetNumber, alt_u8 sendLen, alt_u8 * data);
alt_u8 parseReadDataBlock(alt_u8 * data);
void releaseCard(alt_u8 targetNumber);
bool parseCard();
void setParameters();
void commandDataSend(alt_u8 command[], alt_u8 len);
void selectCard(alt_u8 targetNumber);
bool parseSelectCard();
void commandRFConfig();
void commandRFRtyConfig();
void commandRFFieldConfig();
void commandRFRetriesConfig();
void commandInATR(alt_u8 targetNumber);
void commmandInPSL(alt_u8 targetNumber);
void commandInDEP();
void commandGeneralStatus();

void diagnose(alt_u8 testID, alt_u8* inParams, alt_u8 inParamsLen);
bool parseCommunicatonTest(alt_u8* outParams, alt_u8 inParamsLen);
void getFirmwareVersion();
bool parseFirmwareVersion(FirmwareVersion* firmware);
void samConfiguration(alt_u8 mode, alt_u8 timeout, alt_u8 irqMode);
void inListPassiveTarget(alt_u8 maxTargets, alt_u8 cardType);
bool parsePassiveTargetData(alt_u8* numTargets, TargetData* targetData);
bool checkAck();
bool checkGeneralSuccess();

#endif /* PN532_H_ */
